//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     rvars.h
// Purpose:    The set of variables in a rule.
// Last Change:27.09.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2000-2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	RVARS_INCLUDED
#define RVARS_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef VAR_INCLUDED
#include "var.h"
#endif

#ifndef RVAR_INCLUDED
#include "rvar.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// RVAR_MAGIC: Magic number (identifies objects of this class for debugging).
//-----------------------------------------------------------------------------

#ifdef VER_DEBUG
static const long RVARS_MAGIC = 0x52565300L;	// 'RVS\0'
#endif

//=============================================================================
// Class for the Set of Variables in a Rule:
//=============================================================================

class rvars_c {

	public:

//-----------------------------------------------------------------------------
// Methods:
//-----------------------------------------------------------------------------

	// Constructor:
	rvars_c(void);

	// Destructor:
	~rvars_c(void);

	// Find variable in this set, return its offset (-1 if not found):
	int find_var(var_t var) const;

	// Add a variable to this set, return its offset:
	int add_var(var_t var);

	// Return number of variables in the current set:
	inline int get_num_vars(void) const
		{ CHECK(valid(), "rvars_c::get_num_vars: Bad object");
		  return(NumVars); }

	// Return pointer to first variable:
	inline rvar_t get_first(void) const
		{ CHECK(valid(), "rvars_c::get_first: Bad object");
		  return(First); }

	// Return variable with given offset:
	var_t get_var(int offset) const;

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity Check:
	public:
	bool_t	valid(void) const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;	// Must be "RVARS_MAGIC"
#endif

//-----------------------------------------------------------------------------
// Private Members:
//-----------------------------------------------------------------------------

	private:

	// First: Pointer to first variable.
	rvar_t First;

	// Last: Pointer to currently last variable.
	rvar_t Last;

	// NumVars: Number of variables in this set.
	int NumVars;

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef rvars_c *rvars_t;

//=============================================================================
// End of Include File:
//=============================================================================

#endif
